function[Result]=glm(Y,X,distribution,link_function,dispersion_parameter,constant,residual_type,maxiter,accuracy)

//INPUT PARAMETERS CHECK UP

inputpar=argn(2);
if inputpar < 3
	error('Error in input parameters: glm requires at least three input parameters.')
end
if inputpar==8
	accuracy=0.001;
end
if inputpar==7
	maxiter=1000;accuracy=0.001;
end
if inputpar==6
	residual_type='pearson';maxiter=1000;accuracy=0.001;
end
if inputpar==5
    constant=1;residual_type='pearson';maxiter=1000;accuracy=0.001;
end
if inputpar==4
    if (distribution=='gamma')|(distribution=='normal')|(distribution=='inverse_gaussian')
        dispersion_parameter='mean_deviance';
    elseif (distribution=='poisson')|(distribution=='binomial')
        dispersion_parameter=1;
    end
    constant=1;residual_type='pearson';maxiter=1000;accuracy=0.001;
end
if inputpar==3
    if distribution=='gamma'
        link_function='reciprocal_link';dispersion_parameter='mean_deviance',pw=[];
    elseif distribution=='normal'
        link_function='identity_link';dispersion_parameter='mean_deviance',pw=[];
    elseif distribution=='inverse_gaussian'
        link_function='power_link';pw=-2;dispersion_parameter='mean_deviance';
    elseif distribution=='poisson'
        link_function='logarithmic_link';dispersion_parameter=1,pw=[];
    elseif distribution=='binomial'
        link_function='logit_link';dispersion_parameter=1,pw=[];
    end
    constant=1;residual_type='pearson';maxiter=1000;accuracy=0.001;
end
if (inputpar>=4)
    if (link_function=='power_link')
        printf('Please insert the exponent value for the power_link function:');pw=scanf('%f');
    else
        pw=[];
    end
end
if (distribution=='binomial')
    if((size(Y,2)~=2))
        error('Error in input parameters: Y must be a two columns matrix: the first column for the sample value and the second column for the sample size.')
    end
    if (link_function=='logarithmic_link')|(link_function=='identity_link')|(link_function=='power_link')|(link_function=='reciprocal_link')|(link_function=='squareroot_link')
        error('Error in input parameters: link function not correct in this model.')
    end
else
    if (link_function=='logit_link')|(link_function=='probit_link')|(link_function=='complog_link')
        error('Error in input parameters: link function not usual in this model.')
    end
    if((size(Y,2)~=1))
        error('Error in input parameters: Y must be a one columns matrix.')
    end
end
if((size(Y,1)~=size(X,1)))
	error('Error in input parameters: the number of rows in Y must equal the number of rows in X.')
end

//INPUT PARAMETERS SET UP

[n,p]=size(X);
if p>n
    X=X';[n,p]=size(X);
end
if constant==1
    X=[ones(n,1)';X']';p=p+1;
end
if (distribution=='binomial')
    sample_size=Y(:,2);Y=Y(:,1);
    if(link_function=='logit_link')|(link_function=='complog_link')|(link_function=='probit_link')
        mu=sample_size.*(Y+0.5)./(sample_size+1);
    elseif (link_function=='logarithmic_link')|(link_function=='reciprocal_link')
        mu=Y+(Y==0);
    else
        mu=Y;
    end
else
    sample_size=[];
    if (link_function=='logarithmic_link')|(link_function=='reciprocal_link')
        mu=Y+(Y==0);
    else
        mu=Y;
    end
end
BetaV=zeros(p,maxiter);SE=zeros(p,maxiter);corre=zeros(p,maxiter*p);psi=zeros(1,maxiter);cont=0;preci=replicate(accuracy,[p,1]);W=diag(ones(n,1));
if dispersion_parameter=='mean_deviance'
    psi_value=1;
else
    psi_value=dispersion_parameter;
end
eta=evaluate(mu,sample_size,pw,'eta',link_function);

//ITERATIVELY REWEIGHTED LEAST SQUARE ALGORITHM (IRLS)

for i=1:maxiter
    if((cont>20)|(i==maxiter))
        BetaV(:,i:maxiter)=[];
        SE(:,i:maxiter)=[];
        psi(1,i:maxiter)=[];
        corre(:,((i*p)-(p-1)):(maxiter*p))=[];
        Result.Distribution=distribution;
	  if link_function=='power_link'
		Result.Link_function=strcat([link_function," (",string(pw),")"]);
	  else
		Result.Link_function=link_function;
	  end
        Result.Variables=p-1;
        Result.Observations=n;
        Result.Estimated_Parameters=BetaV(:,size(BetaV,2));
        Result.Standard_Errors=SE(:,size(SE,2));
        Result.Correlation_Matrix=corre(:,(size(corre,2)-(p-1)):size(corre,2));
        Result.Yhat=evaluate(X*Result.Estimated_Parameters,sample_size,pw,'mu',link_function);
        Result.Linear_Predictor=X*Result.Estimated_Parameters;
        Result.Residual=Y-Result.Yhat;
        if distribution=='gamma'
            mu=Result.Yhat+0.00000001*(Result.Yhat==0);yy=Y+(Y==0).*mu;
            if(residual_type=='pearson')
                Result.Residual_Pearson=(Y-Result.Yhat)./(Result.Yhat);
            elseif(residual_type=='deviance')
                Result.Residual_Deviance=(sign(Y-Result.Yhat)).*sqrt(2*((-log(yy./mu)+(Y-mu)./mu)));
            elseif(residual_type=='anscombe')
			    Result.Residual_Anscombe=3*(Y.^(1/3)-Result.Yhat.^(1/3))./Result.Yhat.^(1/3);
		    end
            Result.deviance=2*sum((-log(yy./mu)+(Y-mu)./mu));
            if dispersion_parameter=='mean_deviance'
                Result.Estimated_Dispersion_Parameter=psi(1,size(psi,2));
		    else
      	        Result.Dispersion_Parameter=psi_value;
            end
        elseif distribution=='normal'
            if(residual_type=='pearson')
                Result.Residual_Pearson=Y-Result.Yhat;
		    elseif(residual_type=='deviance')
			    Result.Residual_Deviance=(sign(Y-Result.Yhat)).*sqrt((Y-Result.Yhat).^2);
		    elseif(residual_type=='anscombe')
			    Result.Residual_Anscombe=Y-Result.Yhat;
		    end
            Result.Deviance=sum((Y-Result.Yhat).^2);
            if dispersion_parameter=='mean_deviance'
                Result.Estimated_Dispersion_Parameter=psi(1,size(psi,2));
		    else
      	        Result.Dispersion_Parameter=psi_value;
            end
        elseif distribution=='inverse_gaussian'
            if(residual_type=='pearson')
                Result.Residual_Pearson=(Y-Result.Yhat)./(((Result.Yhat.^3)).^(0.5));
		    elseif(residual_type=='deviance')
			    Result.Residual_Deviance=(sign(Y-Result.Yhat)).*sqrt(((Y-Result.Yhat).^2)./((Result.Yhat.^2).*Y));
		    elseif(residual_type=='anscombe')
			    Result.Residual_Anscombe=(log(Y)-log(Result.Yhat))./Result.Yhat;
		    end
            Result.Deviance=(sum((((Y-Result.Yhat).^2)./((Result.Yhat.^2).*Y))))
            if dispersion_parameter=='mean_deviance'
                Result.Estimated_Dispersion_Parameter=psi(1,size(psi,2));
		    else
      	        Result.Dispersion_Parameter=psi_value;
		    end
        elseif distribution=='poisson'
            yy=Y+0.5*(Y==0);mumu=Result.Yhat+0.5.*(Result.Yhat==0);
		    if(residual_type=='pearson')
			    Result.Residual_Pearson=(Y-Result.Yhat)./(Result.Yhat.^0.5);
		    elseif(residual_type=='deviance')
			    Result.Residual_Deviance=(sign(Y-Result.Yhat)).*sqrt(2.*(Y.*log(yy./mumu)-(Y-mumu)));
		    elseif(residual_type=='anscombe')
			    Result.Residual_Anscombe=((3/2)*(((Y).^(2/3))-((Result.Yhat).^(2/3))))./(Result.Yhat.^(1/6));
		    end
		    Result.Deviance=2*sum((Y.*log(yy./mumu)-(Y-mumu)));
            if dispersion_parameter=='mean_deviance'
                Result.Estimated_Dispersion_Parameter=psi(1,size(psi,2));
		    else
      	        Result.Dispersion_Parameter=psi_value;
		    end
        elseif distribution=='binomial'
            mu=(Result.Yhat)+((Result.Yhat)==0);yfix=(Y)+((Y)==0).*mu;mmmu=(sample_size-mu)+(sample_size==mu);yy=sample_size-(Y);yy=yy+(yy==0).*(mmmu);
            if(residual_type=='pearson')
                Result.Residual_Pearson=(Y-Result.Yhat)./(((Result.Yhat-(Result.Yhat.*(Result.Yhat./sample_size)))).^0.5);
            elseif(residual_type=='deviance')
                Result.Residual_Deviance=(sign(Y-Result.Yhat)).*sqrt(2.*((yfix.*log(yfix./mu))+(yy.*log(yy./mmmu))));
		    elseif(residual_type=='anscombe')
			    beta_value=(gamma(2/3)*gamma(2/3))/gamma(4/3);
			    deff('[yfun]=myfun(xfun)','yfun=(xfun^(2/3-1))*((1-xfun)^(2/3-1))') 
			    for kk=1:length(Y)
				    betainc_valueY(kk)=(1/beta_value)*intg(0,Y(kk)/sample_size(kk),myfun);betainc_valueYhat(kk)=(1/beta_value)*intg(0,Result.Yhat(kk)/sample_size(kk),myfun);
			    end
			    Result.Residual_Anscombe=(beta_value).*(betainc_valueY-betainc_valueYhat)./(max(2.2204e-016,((Result.Yhat./sample_size).*(1-(Result.Yhat./sample_size)))).^(1/6)./sqrt(sample_size));
		    end            
            Result.deviance=2*sum(((yfix.*log(yfix./mu))+(yy.*log(yy./mmmu))));
            if dispersion_parameter=='mean_deviance'
                Result.Estimated_Dispersion_Parameter=psi(1,size(psi,2));
		    else
      	        Result.Dispersion_Parameter=psi_value;
		    end
        end
        Result.Degree_Freedom=n-p;
        Result.t_statistic=(Result.Estimated_Parameters./Result.Standard_Errors);
        if dispersion_parameter=='mean_deviance'
            Result.p_value=2.*cdft("PQ",-abs(Result.t_statistic),replicate(Result.Degree_Freedom,[p,1]));
        else
            Result.p_value=2.*cdfnor("PQ",-abs(Result.t_statistic),replicate(0,[p,1]),replicate(1,[p,1]));
        end
	  Result.Iterations_IRLS_algorithm=i-20;
        return
    else
        delta=ones(n,1)./(evaluate(mu,sample_size,pw,'etamu',link_function));
        if distribution=='gamma'
            answ=(mu.^2)+0.00000001*(mu==0);varianceff=answ.*(answ>0)+(answ<=0); 
        elseif distribution=='normal'
            varianceff=ones(length(mu),1);
        elseif distribution=='inverse_gaussian'
            varianceff=mu.^3;
        elseif distribution=='poisson'
            varianceff=mu+0.00001*(mu==0); 
        elseif distribution=='binomial'
            mu=mu+0.0001*(mu<0.0001);mu=mu-0.0001*(mu>sample_size-0.0001);varianceff=mu-((mu.^2)./sample_size);
        end
        for j=1:n
            W(j,j)=(1/(psi_value*varianceff(j)))*(delta(j))^2;
        end
        z=eta+(((Y)-mu)./delta);R=(X'*W*X)^(-1);RII=R*eye(p,p);BetaV(:,i)=R*X'*W*z;eta=X*BetaV(:,i);mu=evaluate(eta,sample_size,pw,'mu',link_function);
        if dispersion_parameter=='mean_deviance'
            if distribution=='gamma'
                t=(mu>0);psi_value=((1/(n-p)).*(sum(((Y(t)-mu(t)).^2)./((mu(t)).^2))));psi(i)=psi_value;
            elseif distribution=='normal'
                t=(mu>0);psi_value=(1/(n-p)).*(sum((Y(t)-mu(t)).^2));psi(i)=psi_value;
            elseif distribution=='inverse_gaussian'
                t=(mu>0);psi_value=(1/(n-p)).*(sum((((Y(t)-mu(t)).^2)./(mu(t).^3))));psi(i)=psi_value;
            elseif distribution=='poisson'
                t=(mu>0);psi_value=(((1/(n-p)).*(sum(((Y(t)-mu(t)).^2)./((mu(t)))))));psi(i)=psi_value;
            elseif distribution=='binomial'
                t=(mu>0);psi_value=(1/n-p)*sum(((Y(t)-mu(t)).^2)./((mu(t)-(((mu(t).^2))))));psi(i)=psi_value;    
            end
        end
        if i>1
            if ((abs(BetaV(:,i))-abs(BetaV(:,i-1)))<preci)
                cont=cont+1;
            end
        end
        IA=(X'*W*X);                  
        SE(:,i)=(diag(inv(IA))).^(0.5);
        corre(:,((i*p)-(p-1)):(i*p))=RII./(SE(:,i)*SE(:,i)')
    end
end
 
//EVALUATE FUNCTION

function [output]=evaluate(input1,input2,input3,request,linkage)
if (linkage=='logarithmic_link')
   output=logarithmic_link(input1,input2,request);
end  
if (linkage=='complog_link')
   output=complog_link(input1,input2,request);   
end
if (linkage=='power_link')
   output=power_link(input1,input2,input3,request);   
end   
if (linkage=='identity_link')
   output=identity_link(input1,input2,request);
end   
if (linkage=='logit_link')
   output=logit_link(input1,input2,request);
end
if (linkage=='probit_link')
   output=probit_link(input1,input2,request);
end
if (linkage=='reciprocal_link')
   output=reciprocal_link(input1,input2,request);
end  
if (linkage=='squareroot_link')
   output=squareroot_link(input1,input2,request);
end





