.TH glm "December 2003" "S. Favaro" "Scilab Function"
.so ../sci.an
.SH NAME
glm -  generalized linear model function
.SH CALLING SEQUENCE
.nf
[Result]=glm(Y,X,distribution,link_function,dispersion_parameter,constant,
             residual_type,maxiter,accuracy)
.fi
.SH PARAMETERS
.TP 
Y    :  
response variable or dependent variable (n x 1 matrix). In the Binomial models Y is a n x 2 matrix where the first column holds the sample values and the second column holds the samples size.
.TP 
X    :    
covariates matrix or indipendent variables (n x p matrix). n is the number of observations and p is the number of covariates or indipendent variables.
.TP
distribution    : 
distribution for the model (text string value). Acceptable values for distribution parameter are the text strings: 'gamma', 'inverse_gaussian', 'normal', 'poisson' and 'binomial'.
.TP
link_function    :  
link function for the model (text string value). Possible value for link_function parameter are: complog_link, identity_link, logit_link,logarithmic_link, power_link, probit_link, reciprocal_link, squareroot_link. The link function depends on the distribution parameterso that acceptable values for Gamma models, Normal models, Poisson models and Inverse Gaussian models are: identity_link, logarithmic_link,power_link, reciprocal_link and squareroot_link. For Binomial modelsacceptable values are: logit_link, probit_link and complog_link.
.TP
dispersion_parameter    : 
acceptable values for dispersion parameter are: the text string value 'mean_deviance' to estimate the dispersion parameter for the model or a real value to use it like a fixed value for the dispersion parameterfor the model.
.TP
constant    :  
boolean input parameter to include or to omit the constant term in the model. The value 1 includes the constant term in the model, the value 0 omits it.
.TP
residual_type    :  
type of residuals for the model in text string value. Acceptable values for residual_type parameter are: 'pearson' to compute Pearson residuals, 'deviance' to compute deviance residuals and 'anscombe' to compute Anscombe residuals.
.TP
maxiter    :  
maximum number of iterations for the iteratively reweighted least squares algorithm (IRLS).
.TP
accuracy    :  
accuracy in convergence for the iteratively reweighted least squares algorithm (IRLS).
.TP
Result    :
a structure that includes the following fields of output:
.TP
 .Distribution    :
distribution for the model.
.TP
 .Link_function    :
link function for the model.
.TP
 .Variables    :  
number of covariates (number of columns for X matrix).
.TP
 .Observations    :  
number of observations (number of rows for Y matrix).     
.TP
 .Estimated_Parameters    :  
estimated parameters by iteratively reweighted least squares algorithm (IRLS).
.TP
 .Standard_Errors    :  
standar errors for estimated parameters by iteratively reweighted least squares algorithm (IRLS).
.TP
 .Correlation_Matrix    :  
correlation matrix for estimated parameters by iteratively reweighted least squares algorithm (IRLS).
.TP
 .Yhat    :  
fitted values for the model.
.TP
 .Linear_Predictor    :  
linear predictor for the model (usually writter as eta);
.TP
 .Residual    :  
residual for the model (Y-result.Yhat).
.TP
 .Residual_Pearson    :   
if residual_type is 'pearson'.
.TP
 .Residual_Devianve    :   
if residul_type is 'deviance'.
.TP
 .Residual_Anscombe    :   
if residual_type is 'anscombe'.
.TP
 .Deviance    :  
deviance for the model;
.TP
 .Estimated_Dispersion_Parameter    :  
fixed or estimated dispersion parameter for the model.
.TP
 .Degree_Freedom    :  
degrees of freedom for error;
.TP
 .t_statistic    :  
t statistics for model's estimated parameters.
.TP
 .p_value    :  
p value for model's estimated parameters.
.TP
 .Iteration_IRLS_algorithm    :  
number of iterations to reach the convergence for the iteratively reweighted least squares algorithm (IRLS). 	
.SH DESCRIPTION
glm function is the main function of the glmbox SCILAB package produced to analyse some of most popular statistical models belonging to wide class of the generalized linear models (GLM) such as Binomial models, Poisson models, Normal models Gamma models and Inverse Gaussian models.
.nf
[Result]=glm(Y,X,distribution,link_function,dispersion_parameter,
             constant,residual_type,maxiter,accuracy)

[Result]=glm(Y,X,distribution) 
.fi
.TP
The first version of glm function provides more control for the input parameters; the second version of glm function provides a set of default parameters which are usually used for the distribution chosen
.TP
for Gamma distribution the default parameters
.nf
link_function=reciprocal_link;
dispersion_parameter='mean_deviance';
constant=1;
residual_type='pearson';
maxiter=1000;
accuracy=0.001.
.fi
.TP
for Inverse Gamma distribution the default parameters
.nf 
link_function=power_link (-2);
dispersion_parameter='mean_deviance';
constant=1;
residual_type='pearson';
maxiter=1000;
accuracy=0.001.
.fi
.TP
for Normal distribution the default parameters 
.nf
link_function=identity_link;
dispersion_parameter='mean_deviance';
constant=1;
residual_type='pearson';
maxiter=1000;
accuracy=0.001;
.fi
.TP
for Poisson distribution the default parameters
.nf 
link_function=logarithmic_link;
dispersion_parameter=1;
constant=1;
residual_type='pearson';
maxiter=1000;
accuracy=0.001;
.fi
.TP
for Binomial distribution the default parameters 
.nf
link_function=logit_link;
dispersion_parameter=1;
constant=1;
residual_type='pearson';
maxiter=1000;
and accuracy=0.001;
.fi
.TP 0
References:
.TP
[1] Dobson, A.J. (1990), "An Introduction to Generalized Linear Models", CRC Press.
.TP
[2] Gill, J. (2001), "Generalized Linear Models: a unified approach", (quantitative applications in the social sciences, 134). Sage Pubblications
.TP
[3] McCullagh, P., and J.A. Nelder (1990), "Generalized Linear Models", CRC Press.
.TP
[4] Myers, R.M., Montgomery, D.C. and Vining, G.G. (2001), "Generalized Linear Models with applications in engineering and the science", John Wiley and Sons.
.SH SEE ALSO
complog_link, identity_link, reciprocal_link, logarithmc_link, power_link, squareroot_link, logit_link, probit_link.
.SH AUTHOR
Created for Scilab by S. Favaro (Email: sfavaro@katamail.com) 20-12-03.

